@@ -107,10 +107,16 @@ class CachingResolver(lxml.etree.Resolver):
         if request == u"internal:/rfc.number":
             if self.rfc_number:
                 return self.resolve_string(self.rfc_number, context)
+            return self.resolve_string("XXXX", context)
         if not urlparse(request).netloc:
             # Format the request from the relative path of the source so that 
             # We get the exact same path as in the XML document
-            request = os.path.relpath(request, self.source_dir)
+            if request.startswith("file:///"):
+                request = request[8:]
+            try:
+                request = os.path.relpath(request, self.source_dir)
+            except ValueError:
+                pass
         path = self.getReferenceRequest(request)
         return self.resolve_filename(path, context)
     
@@ -403,9 +409,12 @@ class XmlRfcParser:
 
         # Get hold of the rfc number (if any) in the rfc element, so we can
         # later resolve the "&rfc.number;" entity.
-        for action, element in context:
-            if element.tag == "rfc":
-                self.rfc_number = element.attrib.get("number", None)
+        try:
+            for action, element in context:
+                if element.tag == "rfc":
+                    self.rfc_number = element.attrib.get("number", None)
+        except lxml.etree.XMLSyntaxError:
+            self.rfc_number = None
 
         # now get a regular parser, and parse again, this time resolving entities
         parser = lxml.etree.XMLParser(dtd_validation=False,
@@ -513,7 +522,7 @@ class XmlRfc:
             "refparent":	"References",
             "rfcedstyle":	"no",
             "rfcprocack":	"no",
-            "sectionorphan":    5,
+            "sectionorphan":    4,
             "slides":		"no",
             "sortrefs":		"no",
             "strict":		"no",
