@@ -203,6 +203,15 @@ def formatXmlWhitespace(tree):
                                re.sub('\.\s*\n\s*', '.  ', \
                                element.tail))
 
+def ascii_split(text):
+    """ We have unicode strings, but we want to split only on the ASCII
+        whitespace characters so that nbsp does not get split.
+    """
+
+    if isinstance(text, type('')):
+        return text.split()
+    return re.split("[ \t\n\r\f\v]+", text)
+
 
 # ----------------------------------------------------------------------
 # Base conversions.
@@ -304,7 +313,7 @@ def _replace_unicode_characters(str):
         &wj; &zwsp; &nbsp; &nbhy;
     """
     while True:
-        match = re.search(u'([^ -\x7e\u2060\u200B\u00A0\u2011])', str)
+        match = re.search(u'([^ -\x7e\u2060\u200B\u00A0\u2011\r\n])', str)
         if not match:
             return str
         if match.group(1) in _unicode_replacements:
