@@ -300,6 +300,7 @@ class BaseRfcWriter:
         self.expire_string = ''
         self.ascii = False
         self.nbws_cond = u'\u00A0'
+        self.eref_list = []
 
         # We will refer to the XmlRfc document root as 'r'
         self.xmlrfc = xmlrfc
@@ -311,6 +312,7 @@ class BaseRfcWriter:
         self.refs_start = 1             # Start of references sections
         self.figure_count = 0
         self.table_count = 0
+        self.eref_count = 0
 
         # Set RFC number and draft flag
         self.rfcnumber = self.r.attrib.get('number', '')
@@ -322,7 +324,7 @@ class BaseRfcWriter:
         # Item Indicies
         self._index = []
         self._iref_index = {}
-        
+
     def _make_iref(self, item, subitem=None, anchor=None):
         """ Create an iref ID object if it doesnt exist yet """
         last = None
@@ -409,6 +411,14 @@ class BaseRfcWriter:
         self._index.append(item)
         return item
 
+    def _indexCref(self, counter, anchor):
+        counter = str(counter)
+        autoName = 'Comment' + self.nbws_cond + anchor
+        autoAnchor = 'rfc.comment.' + counter
+        item = _RfcItem(autoName, autoAnchor, anchor=anchor, toc=False)
+        self._index.append(item)
+        return item
+
     def get_initials(self, author):
         """author is an rfc2629 author element.  Return the author initials,
         fixed up according to current flavour and policy."""
@@ -794,7 +804,9 @@ class BaseRfcWriter:
         for element in section:
             # Check for a PI
             if element.tag is lxml.etree.PI:
-                self.xmlrfc.parse_pi(element)
+                pis = self.xmlrfc.parse_pi(element)
+                if pis and "needLines" in pis:
+                    self.needLines(pis["needLines"])
             # Write elements in XML document order
             if element.tag == 't':
                 anchor = element.attrib.get('anchor')
@@ -817,6 +829,8 @@ class BaseRfcWriter:
         # Recurse on sections
         for child_sec in section.findall('section'):
             if appendix == True and not count_str:
+                if s_count == 1 and self.pis["rfcedstyle"] == "yes":
+                   self.needLines(-1)
                 # Use an alphabetic counter for first-level appendix
                 uppercase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                 self.write_section_rec(child_sec, uppercase[s_count - 1],
@@ -947,6 +961,7 @@ class BaseRfcWriter:
         self.refs_start = 1             # Start of references sections
         self.figure_count = 0
         self.table_count = 0
+        self.eref_count = 0
 
         # Middle sections
         middle = self.r.find('middle')
@@ -960,18 +975,22 @@ class BaseRfcWriter:
         references = self.r.findall('back/references')
         # Write root level references header
         refs_title = self.pis['refparent']
-        if len(references) == 1:
+        if len(references) == 1 and not self.eref_list:
             refs_title = references[0].attrib.get('title', refs_title)
 
         if len(references) > 0:
             self._indexReferences(refs_counter, title=refs_title)
 
-        if len(references) > 1:
+        if len(references) > 1 or self.eref_list:
             for i, reference_list in enumerate(references):
                 refs_newcounter = refs_counter + '.' + str(i + 1)
                 refs_title = reference_list.attrib.get('title', self.pis["refparent"])
                 self._indexReferences(refs_newcounter, title=refs_title, \
                                       subCounter=i+1, level=2)
+            if self.eref_list:
+                refs_newcounter = refs_counter + '.' + str(len(references)+1)
+                self._indexReferences(refs_newcounter, title="URIs", level=2, subCounter=len(references)+1)
+
         for reference_list in references:
             for ref in reference_list:
                 if len(ref):
@@ -979,6 +998,8 @@ class BaseRfcWriter:
                     title = ref.find("front/title").text
                     self._indexRef(ref_counter, title=title, anchor=ref.attrib["anchor"])
 
+            
+
         # Appendix sections
         back = self.r.find('back')
         if back is not None:
@@ -1011,6 +1032,7 @@ class BaseRfcWriter:
         self.refs_start = 1             # Start of references sections
         self.figure_count = 0
         self.table_count = 0
+        self.eref_count = 0
 
         # Block header
         topblock = self.pis['topblock']
@@ -1070,10 +1092,10 @@ class BaseRfcWriter:
         references = self.r.findall('back/references')
         # Write root level references header
         refs_title = self.pis['refparent']
-        if len(references) == 1:
+        if len(references) == 1 and not self.eref_list:
             refs_title = references[0].attrib.get('title', refs_title)
 
-        if len(references) > 0:
+        if len(references) > 0 or self.eref_list:
             self.write_heading(refs_title, bullet=refs_counter + '.', \
                                autoAnchor='rfc.references')
         if len(references) > 1:
@@ -1087,11 +1109,16 @@ class BaseRfcWriter:
         elif len(references) == 1:
             self.write_reference_list(references[0])
 
+        if self.eref_list:
+            self.write_erefs(refs_counter, len(references)+1)
+
         # Appendix sections
         back = self.r.find('back')
         if back is not None:
             self.write_section_rec(back, None, appendix=True)
 
+        self.write_crefs()
+
         # Index section, disable if there are no irefs
         if len(self._iref_index) > 0:
             self.insert_iref_index()
@@ -1147,6 +1174,16 @@ class BaseRfcWriter:
         if not self.quiet and filename:
             xml2rfc.log.write('Created file', filename)
 
+    def write_erefs(self, refs_counter, refs_subsection):
+        """ Only text versions do this so provide a default that does nothing
+        """
+        pass
+
+    def write_crefs(self):
+        """ Only text versions do this so provide a default that does nothing
+        """
+        pass
+
     def check_for_unused_references(self):
         """ If this is a reference and it is not used - then warn me
         """
@@ -1154,6 +1191,10 @@ class BaseRfcWriter:
             for item in self._index:
                 if item.autoAnchor.startswith("rfc.ref.") and not item.used:
                     xml2rfc.log.warn("no <xref> in <rfc> targets <reference anchor='%s'>" % item.anchor)
+
+    def needLines(self, count):
+        """ Deal with the needLines PI """
+        pass
                 
     # -----------------------------------------
     # Base writer interface methods to override
@@ -1182,7 +1223,7 @@ class BaseRfcWriter:
         raise NotImplementedError('write_title() needs to be overridden')
 
     def write_heading(self, text, bullet='', autoAnchor=None, anchor=None,
-                      level=1):
+                      level=1, breakNeed=0):
         """ Writes a section heading """
         raise NotImplementedError('write_heading() needs to be overridden')
 
